/*****************
 * "config.hpp"
 *
 *  Copyright 2013
 *
 *  This file is part of Actuation.
 *
 *  Actuation is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Actuation is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Actuation.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Description:
 *
 *
 *****************/

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "include.h"

class Config
{
private:
  bool processCommand(std::string command, std::string value);

  /* Settings */
  int dispense_steps_;

  int max_height_; //defines intial zoom level
  int max_width_;

  //max values
  int max_steps_;
  //SVG parsing settings
  //these work on all SVGs tested so far (that do not contain arcs)
  //TODO: store these in a config file or allow them to be changed within the program

  //all electrode coordinates are scaled by this value before performing intersection calculations
  // 5 works for all devices currently
  //decrease if not all intersections are found
  //increase if incorrect intersections are found
  double multiple_;

  //these are similar are used in different cases
  //DO NOT CHANGE IF USING THE SAME CONNECTORS (the problem is elsewhere)
  double tolerance_; //the max variation in the vertical position of the connectors
  double precision_; //the max height difference among connectors

  double prec_; //the amount by which to extend line segments when checking intersection
  double div_; //used when checking whether the endpoints of line segments are the same

  int debug_;
  std::string experiment_logs_dir_;
  std::string devices_dir_;
  std::string procedures_dir_;
  std::string device_path_;

  boost::filesystem::path profile_path_;
  static const char* class_name_;
public:
  Config();
  ~Config();
  bool read(const char* profile_path);
  int dispense_steps(){return dispense_steps_;};
  int max_height(){return max_height_;};
  int max_width(){return max_width_;};
  int max_steps(){return max_steps_;};
  double multiple(){return multiple_;};
  double tolerance(){return tolerance_;};
  double precision(){return precision_;};
  double prec(){return prec_;};
  double div(){return div_;};
  int debug(){return debug_;};
  const char* experiment_logs_dir(){return experiment_logs_dir_.c_str();};
  const char* devices_dir(){return devices_dir_.c_str();};
  const char* procedures_dir(){return procedures_dir_.c_str();};
  bool set_device_path(const char* device_path);
  const char* device_path(){return device_path_.c_str();};
};

#endif
