/*****************
 * "electrode_set.cpp"
 *
 *  Copyright 2014 Steve Shih
 *
 *  This file is part of Actuation.
 *
 *  Actuation is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Actuation is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Actuation.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Description:
 *
 *  Contains functions for creating and using a set of electrodes.
 *
 *****************/
#include "electrode_set.h"

using namespace std;

//This function takes the electrodes parsed from the SVG file
//and classifies them by type, sorts them, and determines which
//of them are connected
//(the worst piece of code I have ever written)
//TODO: rewrite this function
ElectrodeSet::ElectrodeSet(Electrode ** electrodes)
{
  if(electrodes == NULL)
    {
      Logging::LogError("The source array is empty.", "ElectrodeSet", "ElectrodeSet");
      return;
    }

  int i;
  for (i = 0; electrodes[i]->get_type() > -1; i++)
    {
      array.push_back(electrodes[i]);
    }

  int electrode_i = 0;
  int point_i;
  point cp;
  //find the top leftmost and bottom leftmost electrode
  point top_left = find_top_left_point();
  point bottom_left = find_bottom_left_point();

  //mark the array as being either connectors, array, or wires
  //-1 - end of array
  // 0 - wire
  // 1 - electrode
  // 2 - connector
  for(electrode_i = 0; electrode_i < size(); electrode_i++)
    {
      if (array[electrode_i]->get_type())
        {
	  for(point_i = 0; point_i < array[electrode_i]->size(); point_i++)
            {
	      cp = array[electrode_i]->getPoint(point_i);
	      if (30 < abs(top_left.x - bottom_left.x))
                {
		  if (top_left.x < bottom_left.x &&
		      cp.y - configuration.tolerance() < top_left.y)
                    {
		      array[electrode_i]->set_type(CONNECTOR);
		      array[electrode_i]->set_position(TOP);
                    }
		  else if (cp.y + configuration.tolerance() > bottom_left.y)
                    {
		      array[electrode_i]->set_type(CONNECTOR);
		      array[electrode_i]->set_position(BOTTOM);
                    }
                }
	      else if (cp.y - configuration.tolerance() < top_left.y)
                {
		  array[electrode_i]->set_position(TOP);
		  array[electrode_i]->set_type(CONNECTOR);
                }
	      else if (cp.y + configuration.tolerance() > bottom_left.y)
                {
		  array[electrode_i]->set_position(BOTTOM);
		  array[electrode_i]->set_type(CONNECTOR);
                }
            }
        }
    }
  Electrode * list = new Electrode();
  for(electrode_i = 0; electrode_i < size(); electrode_i++)
    {
      if (array[electrode_i]->get_type() == CONNECTOR)
	{
	  list->addElectrode(array[electrode_i]);
	}
    }

  //number the sorted connectors
  int connector_i = 0;
  Electrode * temp=list->next;
  delete list;
  list = temp;
  while (list != NULL && list->get_position() == BOTTOM)
    {
    //if (connector_i % 2 ==0)
	//list->addNumber((3+connector_i)/2);
  //    if (connector_i > 0)
 //   cout << array.size() << "\n";
      list->addNumber(connector_i+1);
      connector_i ++;
      list = list->next;
    }
  connector_i = 1;
  while (list != NULL && list->get_position() == TOP)
    {
     // if (connector_i % 2 == 1)
//	list->addNumber((2 + connector_i)/2 + 20);
   //   cout << array.size() << "\n";
      list->addNumber(connector_i+40);
      connector_i ++;
      list = list->next;
    }

  double lastClock = (double)clock() / (double)CLOCKS_PER_SEC;

  //match the wires to the connectors
  for(electrode_i = 0; electrode_i < size(); electrode_i++)
    {
      if (array[electrode_i]->get_type() == WIRE)
	{
	  //check the first beginning point of the wire
	  cp = array[electrode_i]->frontPoint();
	  for (int i = 0; i < size(); i++)
	    {
	      if (array[i]->get_type() == CONNECTOR &&
		  !array[i]->isEmpty() &&
		  array[i]->check_connection(cp)){
		array[electrode_i]->addIndex(i);
		array[i]->addIndex(electrode_i);
	      }
	    }

	  //check the endpoint of the wire
	  cp = array[electrode_i]->backPoint();
	  for (int i = 0; i < size(); i++)
	    {
	      if (array[i]->get_type() == CONNECTOR &&
		  !array[i]->isEmpty() &&
		  array[i]->check_connection(cp)){
		array[electrode_i]->addIndex(i);
		array[i]->addIndex(electrode_i);
	      }
	    }

	}
    }


  for(int i = 0; i < size(); i++)
    array[i]->multiply(configuration.multiple());
  //match wires to unidentified wires
  for(electrode_i = 0; electrode_i < size(); electrode_i++)
    {
      if (array[electrode_i]->get_type() == WIRE)
	{
	  for (int i = 0; i < size(); i++)
	    {
	      if (i != electrode_i && array[i]->get_type() == WIRE)
		{
		  if (array[electrode_i]->check_wire_intersection(array[i]))
		    {
		      array[i]->addIndex(electrode_i);
		      array[electrode_i]->addIndex(i);
		    }
		}
	    }
	}
    }
   //match the electrodes to the wires
  for(electrode_i = 0; electrode_i < size(); electrode_i++)
    {
      if (array[electrode_i]->get_type() == WIRE)
	{
	  for(point_i = 0; point_i < array[electrode_i]->size(); point_i++)
	    {
	      int i;
	      cp = array[electrode_i]->getPoint(point_i);
	      for (i = 0; i < size(); i++)
		{
		  if (array[i]->get_type() == ELECTRODE&&
		      point_i > 0&&
		      array[i]->check_intersection(cp, array[electrode_i]->getPoint(point_i - 1)))
		    {
		      array[i]->addIndex(electrode_i);
		      array[electrode_i]->addIndex(i);
		    }
		  if (array[i]->get_type() == ELECTRODE&&
		      array[i]->check_connection(cp)){
		    array[i]->addIndex(electrode_i);
		    array[electrode_i]->addIndex(i);
		  }
		}
	    }
	}
    }
  //recursively find the indexes of all electrodes connected to each electrode
  for(int i = 0; i < size(); i++)
    this->fillSet(-1, i);

  //assign numbers to each electrode based on the connectors it is connected to
  for(int i = 0; i < size(); i++)
    for(int ni = 0; ni < array[i]->indexSetSize(); ni++)
      if(array[i]->get_type()!=CONNECTOR&&array[array[i]->getIndex(ni)]->get_type() == CONNECTOR)
	array[i]->addNumber(array[array[i]->getIndex(ni)]->getFirstNumber());

  //for(int i = 0; i < 41; i++)
    for(int i = 0; i < 81; i++)
    connectors.push_back(new Electrode());

  //place the connectors in a separate array
  for(int i = 0; i < size(); i++)
    if(array[i]->get_type() == CONNECTOR)
      connectors[array[i]->getFirstNumber()] = array[i];

  cout<<"time: " <<(double)clock()/(double)CLOCKS_PER_SEC - lastClock<<" second(s)" <<endl;
}

//add the index set of source to dest, recursively repeat for all the electrodes in the
//index set of dest
void ElectrodeSet::fillSet(int source, int dest)
{
  bool changed = true;
  if(source>=0)
    changed = array[dest]->addSet(array[source]->begin(), array[source]->end());
  if(!changed||source==dest)
    return;
  for(int i = 0; i < array[dest]->indexSetSize(); i++)
    fillSet(dest, array[dest]->getIndex(i));
  return;
}

//causes a crash?
ElectrodeSet::~ElectrodeSet()
{
  array.clear();
}

//change the state of all electrodes with the matching number
void ElectrodeSet::changeStateN(int number, int step)
{
  connectors[number]->changeState(step);
}

//change the state of an indexed electrode's connectors
void ElectrodeSet::changeStateI(int index, int step)
{
  setStateI(!getStateI(index, step), index, step);
}

/**
 * Set the state of a connector.
 * \param new_state 0 for ground and 1 for high voltage
 * \param number the number of the connector (1-80)
 * \param step the current step in the procedure
 */
void ElectrodeSet::setStateN(int new_state, int number, int step)
{
  if(new_state!=connectors[number]->get_state(step))
    connectors[number]->changeState(step);
}

void ElectrodeSet::setStateI(int new_state, int index, int step)
{
  if(index<0 || index>size())
    return;
  for(int n = 0; n < array[index]->numberSetSize(); n++)
    if(connectors[array[index]->getNumber(n)]->get_state(step)!= new_state)
      changeStateN(array[index]->getNumber(n), step);
}

//change the numbers of all electrodes in a group
void ElectrodeSet::changeNumberI(int index, std::string new_number)
{
  array[index]->setNumberString(new_number);
}

void ElectrodeSet::incrementRow(int row, int increment)
{
  for(int i = 0; i < size(); i++)
    array[i]->incrementRow(row, increment);
}
/**
 * Return the state of the connector with the given number.
 * \return 0 for ground and 1 for high voltage
 * \param number the number of the connector (1-40)
 * \param step the current step in the procedure
 */
int ElectrodeSet::getStateN(int number, int step)
{
  return connectors[number]->get_state(step);
  return 0;
}

int ElectrodeSet::getStateI(int index, int step)
{
  int state = 0;
  for(int n = 0; n < array[index]->numberSetSize(); n++)
    state = state||getStateN(array[index]->getNumber(n), step);
  return state;
}

int ElectrodeSet::getClosestI(int index, int * dispense_electrodes)
{
  int closest = -1;
  double distance = 9000000;
  double current_distance;
  point index_center = findCenterI(dispense_electrodes[index]);
  for(int i = 0; i < size(); i++)
    {
      point current_center = findCenterI(i);
      current_distance = (current_center.x - index_center.x)*(current_center.x - index_center.x)+
	(current_center.y - index_center.y)*(current_center.y - index_center.y);
      if(i != dispense_electrodes[index]&&
	 array[i]->get_type() == ELECTRODE&&
	 current_distance < distance){
	if(index == 0 || i != dispense_electrodes[index-1]){
	  distance = current_distance;
	  closest = i;

	}
      }
    }

  return closest;
}

//
point ElectrodeSet::findCenterI(int index)
{
  return array[index]->find_center();
}

point ElectrodeSet::find_top_left()
{
  point top_left = {100000000, 100000000};
  point current;
  for(int i = 0; i < size(); i++)
    {
      current = array[i]->find_top_left();
      if(current.x <= top_left.x)
	top_left.x = current.x;
      if(current.y <= top_left.y)
	top_left.y = current.y;
    }
  return top_left;
}

point ElectrodeSet::find_top_left_point()
{
  point top_left = {100000000, 100000000};
  point current;
  for(int i = 0; i < size(); i++)
    {
      current = array[i]->find_top_left();
      if(current.y <= top_left.y){
	top_left.y = current.y;
	if(current.x <= top_left.x)
	  top_left.x = current.x;
      }
    }
  return top_left;
}

point ElectrodeSet::find_bottom_right()
{
  point bottom_right = {0, 0};
  point current;
  for(int i = 0; i < size(); i++)
    {
      current = array[i]->find_bottom_right();
      if(current.x >= bottom_right.x)
	bottom_right.x = current.x;
      if( current.y >= bottom_right.y)
	bottom_right.y = current.y;
    }
  return bottom_right;
}

point ElectrodeSet::find_bottom_left_point()
{
  point bottom_left = {100000, 0};
  point current;
  for(int i = 0; i < size(); i++)
    {
      current = array[i]->find_bottom_left();
      if(current.y >= bottom_left.y){
	bottom_left.y= current.y;
	if(current.x <= bottom_left.x)
	  bottom_left.x = current.x;
      }
    }
  return bottom_left;
}

int ElectrodeSet::get_left(int index)
{
  double limit = find_top_left().x;
  point cp = findCenterI(index);
  while(cp.x > limit)
    {
      for(int i = 0; i < size(); i++)
	{
	  if(i!=index&&array[i]->get_type()==ELECTRODE&&checkConnectionI(i, cp))
	    return i;
	}
      cp.x--;
    }
  return -1;
}
int ElectrodeSet::get_right(int index)
{
  double limit = find_bottom_right().x;
  point cp = findCenterI(index);
  while(cp.x < limit)
    {
      for(int i = 0; i < size(); i++)
	{
	  if(i!=index&&array[i]->get_type()==ELECTRODE&&checkConnectionI(i, cp))
	    return i;
	}
      cp.x++;
    }
  return -1;
}
int ElectrodeSet::get_top(int index)
{
  double limit = find_top_left().y;
  point cp = findCenterI(index);
  while(cp.y > limit)
    {
      for(int i = 0; i < size(); i++)
	{
	  if(i!=index&&array[i]->get_type()==ELECTRODE&&checkConnectionI(i, cp))
	    return i;
	}
      cp.y--;
    }
  return -1;
}
int ElectrodeSet::get_bottom(int index)
{
  double limit = find_bottom_right().y;
  point cp = findCenterI(index);
  while(cp.y < limit)
    {
      for(int i = 0; i < size(); i++)
	{
	  if(i!=index&&array[i]->get_type()==ELECTRODE&&checkConnectionI(i, cp))
	    return i;
	}
      cp.y++;
    }
  return -1;
}

void ElectrodeSet::print()
{
  cout<<"size: "<<size()<<endl;
  for(int i = 0; i < size(); i++)
    array[i]->print();
}

//find the number of the last electrode
int ElectrodeSet::last_num()
{
  int lastnum = 0;
  for(int i = 0; i < size(); i++)
    {
      for(int nIndex = 0; nIndex < array[i]->numberSetSize(); nIndex++)
	{
	  if (array[i]->getNumber(nIndex) > lastnum && array[i]->get_type() == ELECTRODE)
	    lastnum = array[i]->getNumber(nIndex);
	}
    }
  return lastnum;
}

void ElectrodeSet::draw(cairo_t * cr, double scale, point top_left, int current_step, int selected, bool do_show_numbers, bool do_show_keys)
{
  int ssi;
  //draw filled shapes first (electrodes and connectors)
  for (ssi = 0; ssi < size(); ssi++)
    {
      if(array[ssi]->get_type()>0){
	if(selected>-1 && selected == ssi)
	  array[ssi]->draw_selected(cr, scale, top_left);
	else
	  array[ssi]->draw(cr, scale, top_left, getStateI(ssi, current_step));
      }
    }
  //draw lines (wires)
  for (ssi = 0; ssi < size(); ssi++)
    {
      if(array[ssi]->get_type()==0)
	array[ssi]->draw(cr, scale, top_left, 0);
    }
  cairo_set_source_rgb(cr, 0.4, 0.4, 0.4);
  cairo_stroke(cr);


  //number and add keyboard shorcuts over the electrodes
  cairo_set_source_rgb(cr, 1, 1, 1);
  for (ssi = 0; ssi < size(); ssi++)
    {
      if (array[ssi]->get_type()>0 && array[ssi]->numberSetSize() > 0)
	{
	  char buff1[32] = "\0";
	  char buff2[32] = "\0";
	  if(do_show_numbers)
	    sprintf(buff1, "%s", (array[ssi]->getNumberString()).data());
	  if(do_show_keys)
	    sprintf(buff2, " %s",(array[ssi]->getCharString()).data());

	  cairo_move_to(cr,
			scale * (findCenterI(ssi).x - top_left.x) - 7,
			scale * (findCenterI(ssi).y - top_left.y));
	  cairo_show_text(cr, buff1);
	  cairo_move_to(cr,
			scale * (findCenterI(ssi).x - top_left.x) - 7,
			scale * (findCenterI(ssi).y - top_left.y) + 10);
	  cairo_show_text(cr, buff2);
	}
    }
}

void ElectrodeSet::copy_step(int source, int dest)
{
  for(int i = 0; i < size(); i++)
    array[i]->copy_step(source, dest);
}
