/*****************
 * "electrode_set.hpp"
 *
 *  Copyright 2014
 *
 *  This file is part of Actuation.
 *
 *  Actuation is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Actuation is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Actuation.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Description:
 *
 *
 *
 *****************/
#ifndef _ELECTRODE_SET_H_
#define _ELECTRODE_SET_H_

#include "include.h"
#include "electrode.h"

extern Config configuration;

class ElectrodeSet
{
private:
  std::vector<Electrode*> array;
  //std::vector<Electrode*> electrodes;
  std::vector<Electrode*> connectors;

  void fillSet(int source, int dest);

 public:
  ElectrodeSet(Electrode ** electrodes);
  ~ElectrodeSet();

  int size(){return array.size();};

  /* Note: functions ending in N take the number of an electrode as input
   * functions ending in I take the index
   */

  int getFirstNumber(int index){return getNumber(index, 0);};
  int getNumber(int index, int nIndex){return array[index]->getNumber(nIndex);};
  int getNumberSetSizeI(int index){return array[index]->numberSetSize();};
  void addNumber(int index, int new_number){array[index]->addNumber(new_number);};

  void changeNumberI(int selected, std::string new_number);
  void incrementRow(int row, int increment);

  int getIndexN(int number)
  {
    for(int i = 0; i < size(); i++)
      if(array[i]->get_type()==CONNECTOR)
	if(array[i]->isInSet(number))
	  return i;
    return -1;
  }

  std::string getNumberString(int index){return array[index]->getNumberString();};

  int getTypeI(int index){return array[index]->get_type();};
  int getPositionI(int index){return array[index]->get_position();};
  int getStateN(int number, int step);
  int getStateI(int index, int step);

  int getClosestI(int index, int * dispense_electrodes);
  int checkConnectionI(int index, point p){return array[index]->check_connection(p);};

  void changeStateN(int number, int step);
  void changeStateI(int index, int step);
  void setStateN(int new_state, int number, int step);
  void setStateI(int new_state, int index, int step);

  point findCenterI(int index);
  void draw(cairo_t *cr, double scale, point top_left, int current_step, int selected, bool do_show_numbers, bool do_show_keys);
  void print();
  void clear_step(int step)
  {
    for(int i = 0; i < size(); i++)
      array[i]->clear_step(step);
  };
  void clear_all()
  {
    for(int i = 0; i < size(); i++)
      array[i]->clear();
  };

  void copy_step(int source, int dest);

  //find the top leftmost point of all the electrodes
  point find_top_left ();
  point find_top_left_point();
  //find the bottom rightmost point of all the electrodes
  point find_bottom_right ();

  point find_bottom_left_point();

  //find the total number of electrodes
  int last_num();

  //find the electrodes around the selected
  int get_left(int index);
  int get_right(int index);
  int get_top(int index);
  int get_bottom(int index);
};

#endif
